@@ -20,7 +20,7 @@ from subprocess import PIPE, run
 from tempfile import NamedTemporaryFile
 
 from . import Array, Dictionary, Name, Object, Pdf, Stream, Page
-from ._qpdf import _ObjectMapping, Token
+from ._qpdf import _ObjectMapping, Token, PdfError, StreamParser
 from .models import PdfMetadata, Permissions, EncryptionInfo
 
 # pylint: disable=no-member,unsupported-membership-test,unsubscriptable-object
@@ -57,14 +57,20 @@ def augments(cls_cpp):
     elsewhere in the support class or in the target class.
 
     The target class does not have to be C++ or derived from pybind11.
+
+    THIS DOES NOT work for static methods or class methods. pybind11 does not seem
+    to support this sort of runtime modification.
     """
 
     def class_augment(cls, cls_cpp=cls_cpp):
-        for name, fn in inspect.getmembers(cls, inspect.isfunction):
-            fn.__qualname__ = fn.__qualname__.replace(cls.__name__, cls_cpp.__name__)
-            setattr(cls_cpp, name, fn)
-        for name, fn in inspect.getmembers(cls, inspect.isdatadescriptor):
-            setattr(cls_cpp, name, fn)
+        for name, member in inspect.getmembers(cls):
+            if inspect.isfunction(member):
+                member.__qualname__ = member.__qualname__.replace(
+                    cls.__name__, cls_cpp.__name__
+                )
+                setattr(cls_cpp, name, member)
+            elif inspect.isdatadescriptor(member):
+                setattr(cls_cpp, name, member)
 
         def block_init(self):
             # Prevent initialization of the support class
@@ -330,6 +336,8 @@ class Extend_Pdf:
 
         """
 
+        # Can use QPDF::closeInputSource() but that requires libqpdf 9.0.0
+
         EMPTY_PDF = (
             b"%PDF-1.3\n"
             b"1 0 obj\n"
@@ -392,6 +400,45 @@ class Extend_Pdf:
         """
         return EncryptionInfo(self._encryption_data)
 
+    def check(self):
+        """
+        Check if PDF is well-formed.  Similar to ``qpdf --check``.
+
+        Returns:
+            list of strings describing errors of warnings in the PDF
+        """
+
+        class DiscardingParser(StreamParser):
+            def __init__(self):  # pylint: disable=useless-super-delegation
+                super().__init__()  # required for C++
+
+            def handle_object(self, obj):
+                pass
+
+            def handle_eof(self):
+                pass
+
+        problems = []
+
+        try:
+            self._decode_all_streams_and_discard()
+        except PdfError as e:
+            problems.append(e)
+
+        discarding_parser = DiscardingParser()
+
+        for basic_page in self.pages:
+            page = Page(basic_page)
+            try:
+                page.parse_contents(discarding_parser)
+            except PdfError as e:
+                problems.append(e)
+
+        for warning in self.get_warnings():
+            problems.append("WARNING: " + warning)
+
+        return problems
+
     def _attach(self, *, basename, filebytes, mime=None, desc=''):  # pragma: no cover
         """
         Attach a file to this PDF
